/*
 * Mach Operating System
 * Copyright (c) 1992 Carnegie Mellon University
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon
 * the rights to redistribute these changes.
 */
/*
 * Amiga interrupt handler entrypoints, called from m68k/locore.s
 *
 * Contributors:
 *      Bryan Ford
 *      Niklas Hallqvist
 *      Markus Wild
 */

#include <machine/frame.h>
#include <machine/custom.h>
#include <machine/mach_param.h>
#include <machine/custom.h>
#include <machine/cia.h>


void
level_1_int_c (int user)
{
  extern void utxint();

  if (custom.intreqr & INTF_SOFTINT)
    {
      custom.intreq = INTF_SOFTINT;
      softclock ();
    }
  else if (custom.intreqr & INTF_TBE)
    utxint ();
  else
    asm ("illegal;");
}


void
level_2_int_c (int user)
{
  extern void kdintr(void);

  int ints = ciaa.icr;

  custom.intreq = INTF_PORTS;
#if defined (A3000) || defined (GVP11) || defined (A2091)
  amiga_dma_intr (0, 0, 2);
  sbic_intr (0, 2);
#endif
#if later
  if(ints & (1<<3))
    kdintr();
#endif
}

void
level_3_int_c (int user)
{
  if (custom.intreqr & INTF_VERTB)
    {
      /* shouldn't we splsoftclock() if user?  Don't think so, we're already
       * at a higher interrupt level! (says NH) */
      custom.intreq = INTF_VERTB;

/* just reset the interrupt above, the clock has migrated to level 6 */
#if 0
      clock_interrupt (1000000 / HZ, user, user);
#endif
    }
  else
#if 1
    printf("spurious non-VERTB INT3\n");
#else
  ;
#endif
}


void
level_4_int_c (int user)
{
  asm ("illegal;");
}


void
level_5_int_c (int user)
{
  extern void utrint();

  if (custom.intreqr & INTF_RBF)
    utrint ();
  else
    asm ("illegal;");
}


void
level_6_int_c (int user)
{
  int ints = ciab.icr;
  
  if (ints & 0x80)
    {
      /* is CIAB, so reset EXTR */
      custom.intreq = INTF_EXTER;
      
      if (ints & 0x01)
	clock_interrupt (1000000 / HZ, user, user);
      else
	{
	  /* deal with anything else you want to deal with here, perhaps
	     timer B (as profiling timer? This is BSD though;-)) */
	  printf ("spurious CIAB interrupt, icr == $%x\n", ints);
	}
    }
  else
    {
      /* deal with other level 6 interrupts. External boards may cause 
	 int6, here's the place to handle them */
      custom.intena = 0x2000; /* Disable interrupt */
#if 1
      printf ("spurious non-CIAB INT6 %x\n", custom.intreqr);
#endif
    }
}


inline void
setsoftclock (void)
{
  custom.intreq = INTF_SETCLR | INTF_SOFTINT;
}
